@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010-2018 Codership Oy <info@codership.com>
+ * Copyright (C) 2010-2020 Codership Oy <info@codership.com>
  */
 
 /*! @file ring buffer storage class */
@@ -116,6 +116,13 @@ namespace gcache
 
         void set_debug(int const dbg) { debug_ = dbg & DEBUG; }
 
+#ifdef GCACHE_RB_UNIT_TEST
+        ptrdiff_t offset(const void* const ptr) const
+        {
+            return static_cast<const uint8_t*>(ptr) - start_;
+        }
+#endif
+
     private:
 
         static size_t const PREAMBLE_LEN = 1024;
